#ifndef EA_ONTICK_MQH_INCLUDED
#define EA_ONTICK_MQH_INCLUDED
/**********************************************************************************
 * Copyright (C) 2010-2020 Dominik Egert <dominik.egert@freie-netze.de>
 *
 * This file holds the main OnCalculate functions.
 *
 * This file may not be copied and/or distributed without explecit permit.
 * Author Dominik Egert / Freie Netze UG.
 **********************************************************************************
 *
 *  File information
 *  ================
 *
 *  Version: 1.00
 *
 *
 *
*/



//*********************************************************************************************************************************************************/
// Expert Advisor definition and code options (Edit/Add/Configure)
//



/////////////////////////////////////////////////////////////////////////////////////////
//
//  Code execution protection
//

// Code EOL protection
/*
#ifndef LIB_CODE_EOL
#define LIB_CODE_EOL
#define LIB_CODE_EOL_B1     0x5E
#define LIB_CODE_EOL_B2     0xD2
#define LIB_CODE_EOL_B3     0xF3
#define LIB_CODE_EOL_B4     0x80
#define LIB_CODE_EOL_B5     0x80
#define LIB_CODE_EOL_B6     0x80
#define LIB_CODE_EOL_B7     0x80
#define LIB_CODE_EOL_B8     0x80
#define LIB_CODE_EOL_HASH   ""
#endif
*/

//
/////////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////////////
//
//  Set type of weighted price to be used:
//      default is weighted close: (h+l+c+c)/4
//      optionally: weighted: (o+h+l+c)/4
//
#ifndef IND_WEIGHTED_TYPE_OHLC4
    //#define IND_WEIGHTED_TYPE_OHLC4

#endif


//
/////////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////////
//
// Include common library functions
//

// Include lib_common.mqh
#include <bali_lib/lib_common.mqh>

//
/////////////////////////////////////////////////////////////////////////////////////////




/////////////////////////////////////////////////////////////////////////////////////////
//
// Set execution definitions
//

// Timer or tick based execution for position checking
//#define EA_FPL_ONTICK

// Timer or tick based execution for order checking
//#define EA_PO_ONTICK

// Activate and define type of TP flagging
//#define EA_ORD_INIT_TP_AS_EVEN true



//
// END definition and code options
// *********************************************************************************************************************************************************/




//*********************************************************************************************************************************************************/
// User configuration input (Edit/Add)
//

// Inputs
/*
input group "[Wave] Profit settings"
input   uint                pos_profit_fractions            = 0;                    // Set profit zones total (0 = auto or ticks; 19 = max, 1 = off)
input   uint                pos_profit_zone_ticksize        = 100;                  // Set profit zone size in ticks
input   uint                pos_fraction_hysteresis         = 50;                   // Hysteresis in % on switching zones (0 = off)
input   uint                pos_volume_reduction            = 25;                   // Percentage rate at which volume is reduced (0 = auto)
input   uint                pos_sl_recovery_fraction        = 1;                    // Zone to set SL to break even (0 = off, 1 = auto)
input   double              pos_sl_recovery_level           = 7.5;                  // Percentage level of TP to set SL to break even (0 = off)
input   double              pos_vol                         = 0.03;                  // Volume
input   double              pos_profit                      = 2.0;                   // Profit factor
input   double              pos_stop                        = 3.0;                   // Loss factor
*/


// Register managed input variables
#ifdef LIB_USER_INPUT_MQH_INCLUDED
/*
LIB_USER_INPUT_ADD(uint,        pos_profit_fractions,           "Set profit zones total (0 = auto or ticks; 19 = max, 1 = off)");
LIB_USER_INPUT_ADD(uint,        pos_profit_zone_ticksize,       "Set profit zone size in ticks");
LIB_USER_INPUT_ADD(uint,        pos_fraction_hysteresis,        "Hysteresis in % on switching zones (0 = off)");
LIB_USER_INPUT_ADD(uint,        pos_volume_reduction,           "Percentage rate at which volume is reduced (0 = auto)");
LIB_USER_INPUT_ADD(uint,        pos_sl_recovery_fraction,       "Zone to set SL to break even (0 = off, 1 = auto)");
LIB_USER_INPUT_ADD(double,		pos_sl_recovery_level,          "Percentage level of TP to set SL to break even (0 = off)");
LIB_USER_INPUT_ADD(double,		pos_profit,                     "Profit factor");
LIB_USER_INPUT_ADD(double,		pos_stop,                       "Loss factor");
*/
#endif
//
// END User configuration input
//*********************************************************************************************************************************************************/







//*********************************************************************************************************************************************************/
// Return codes
//

// Define error group id
#define EA_ERRGROUP_CODE_SINGAL_EXEC        0x02

// Defined return codes
#define EX_SUCCESS                          LIB_ERROR_CODE(EA_ERRGROUP_CODE_SINGAL_EXEC, 0x00)
#define EX_ERR_VOLUME_FRACTION              LIB_ERROR_CODE(EA_ERRGROUP_CODE_SINGAL_EXEC, 0x01)
#define EX_ERR_NOZONES_DEFINED              LIB_ERROR_CODE(EA_ERRGROUP_CODE_SINGAL_EXEC, 0x02)

// Define message resolver
LIB_ERR_REGISTER_RESOLVER_BEGIN(EA_ERRGROUP_CODE_SINGAL_EXEC)
    LIB_ERR_REGISTER_RESOLVER_MSGCODE(EX_SUCCESS,               "EX_SUCCESS",                   "Operation success. An order has been processed.")
    LIB_ERR_REGISTER_RESOLVER_MSGCODE(EX_ERR_VOLUME_FRACTION,   "EX_ERR_VOLUME_FRACTION",       "Operation aborted. Calcualted volume fraction to small.")
    LIB_ERR_REGISTER_RESOLVER_MSGCODE(EX_ERR_NOZONES_DEFINED,   "EX_ERR_NOZONES_DEFINED",       "Operation aborted. No profit zones defined.")
LIB_ERR_REGISTER_RESOLVER_END(EA_ERRGROUP_CODE_SINGAL_EXEC)


//
// END Return codes
//*********************************************************************************************************************************************************/





//*********************************************************************************************************************************************************/
// Custom signal interpretation function (Edit or add your code)
//

//+------------------------------------------------------------------+
//| OnSignal()                                                       |
//+------------------------------------------------------------------+
const bool OnSignal(const int index, bool& bShort, double& abs_sl, double& abs_tp, double& vol, double& open_price, double& order_cancel_level)
{
    /*********************************************************************************************************************************************************/
    /*                                                                                                                                                       */
    /* OnSignal handler                                                                                                                                      */
    /*                                                                                                                                                       */
    /*    This function is called whenever there is a request for opening a position or creating an order.                                                   */
    /*                                                                                                                                                       */
    /*    Execution is governed by the internal ea update timer, which usually defaults to the current period                                                */
    /*    this EA is run on.                                                                                                                                 */
    /*                                                                                                                                                       */
    /*    If the compiler flag EA_SIGNAL_ONTICK is set (#define EA_PO_ONTICK) this function will be called                                                   */
    /*    on every tick called by the terminal into this EA instance.                                                                                        */
    /*                                                                                                                                                       */

    // Return
    err_SetUserError(EX_SUCCESS);
    return(true);
}


//
// END Custom signal interpretation function 
//*********************************************************************************************************************************************************/




//*********************************************************************************************************************************************************/
// Custom Profit and StopLoss handlers (Edit or add your code)
//

//+------------------------------------------------------------------+
//| OnPendingOrder()                                                 |
//+------------------------------------------------------------------+
const bool OnPendingOrder(const int order_id)
const bool OnPendingOrder(order_position_info& order_details)
{
    /*********************************************************************************************************************************************************/
    /*                                                                                                                                                       */
    /* OnPendingOrder handler                                                                                                                                */
    /*                                                                                                                                                       */
    /*    This function is called for all currently pending orders.                                                                                          */
    /*    Execution is governed by the internal ea update timer, which usually defaults to the current period                                                */
    /*    this EA is run on.                                                                                                                                 */
    /*                                                                                                                                                       */
    /*    If the compiler flag EA_SIGNAL_ONTICK is set (#define EA_PO_ONTICK) this function will be called                                                   */
    /*    on every tick called by the terminal into this EA instance.                                                                                        */
    /*                                                                                                                                                       */




    // Access data on order
    //data::order_list[order_id];




    // Return false will stop execution 
    // of processing current list of tickets
    // Return
    err_SetUserError(EX_SUCCESS);
    return(true);
}



//+------------------------------------------------------------------+
//| OnFloatingPnL()                                                  |
//+------------------------------------------------------------------+
const bool OnFloatingPnL(const int pos_id)
const bool OnFloatingPnL(order_position_info& order_details)
{
    /*********************************************************************************************************************************************************/
    /*                                                                                                                                                       */
    /* OnProfit handler                                                                                                                                      */
    /*                                                                                                                                                       */
    /*    This function is called for every open position.                                                                                                   */
    /*    Execution is governed by the internal ea update timer, which usually defaults to the current period                                                */
    /*    this EA is run on.                                                                                                                                 */
    /*                                                                                                                                                       */
    /*    If the compiler flag EA_SIGNAL_ONTICK is set (#define EA_FPL_ONTICK) this function will be called                                                  */
    /*    on every tick thats been called by the terminal into this EA instance.                                                                             */
    /*                                                                                                                                                       */
    /*                                                                                                                                                       */
    /*    Currently set zone is preserved in the SL value                                                                                                    */
    /*    Performed volume reductions are preserved in the current TP value                                                                                  */
    /*                                                                                                                                                       */
    /*                                                                                                                                                       */


    // Validate position age
    
        const int       pos_age_seconds     = (int)(TimeCurrent() - data::position_list[pos_id].update_time);
        const int       pos_periods_age     = (pos_age_seconds / 60) + (((pos_age_seconds % 60) > NULL) & 0x01);

        if( (pos_periods_age == NULL) 
         || (data::position_list[pos_id].profit_ticks < NULL))
        { return(true); }
        

        // DEV
        const datetime  now         = TimeCurrent();
        const int       index_high  = iHighest(data::position_list[pos_id].symbol, PERIOD_M1, MODE_HIGH, pos_periods_age, NULL);
        const int       index_low   = iLowest(data::position_list[pos_id].symbol, PERIOD_M1, MODE_LOW, pos_periods_age, NULL);
        // END DEV


    // Market infos
    
        const double    symbol_vol_min      = SymbolInfoDouble(data::position_list[pos_id].symbol, SYMBOL_VOLUME_MIN);
        const double    symbol_vol_max      = SymbolInfoDouble(data::position_list[pos_id].symbol, SYMBOL_VOLUME_MAX);


    // Zone geometry 
    
        const double    digit_f             = MathPow(10.0, SymbolInfoInteger(data::position_list[pos_id].symbol, SYMBOL_DIGITS));
        const double    price_granularity   = (1.0 / digit_f);
        const double    short_f             = (1.0 - (2.0 * ((data::position_list[pos_id].type_sell) & 0x01)));
        const int       init_tp_ticks       = (int)MathAbs(MathRound((data::position_list[pos_id].initial_take_profit - data::position_list[pos_id].open_price) * digit_f));
        const int       init_sl_ticks       = (int)MathAbs(MathRound((data::position_list[pos_id].open_price - data::position_list[pos_id].initial_stop_loss) * digit_f));
        const int       current_tp_ticks    = (int)MathAbs(MathRound((data::position_list[pos_id].take_profit - data::position_list[pos_id].open_price) * digit_f));
        const int       current_sl_ticks    = (int)MathAbs(MathRound((data::position_list[pos_id].open_price - data::position_list[pos_id].stop_loss) * digit_f));

        //const double    volume_zone_size    = MathRange(MathRound((data::position_list[pos_id].initial_volume / (double)((init_tp_ticks / init_sl_ticks) + inline_BooleanDouble((init_tp_ticks % init_sl_ticks) > NULL))) - (symbol_vol_min / 2.0), inline_MathDigits(symbol_vol_min)), symbol_vol_min, symbol_vol_max);
		//const int		volume_zones_total	= (int)MathRound(data::position_list[pos_id].initial_volume / volume_zone_size, 0x00);
		const int		volume_zones_max	= (int)MathRound(data::position_list[pos_id].initial_volume / symbol_vol_min, 0x00);
        const int       tp_zones_total      = (init_tp_ticks / init_sl_ticks) + inline_BooleanInteger((init_tp_ticks % init_sl_ticks) > NULL);
		const int		zones_total			= (tp_zones_total * inline_BooleanInteger(tp_zones_total < volume_zones_max)) + (volume_zones_max * inline_BooleanInteger(tp_zones_total >= volume_zones_max));
        const int       zone_size           = init_tp_ticks / zones_total;
        const int       zone_set            = current_sl_ticks * ((data::position_list[pos_id].initial_stop_loss != data::position_list[pos_id].stop_loss) & 0x01);
        const int       zone_offset         = zone_size / 2;

        const double    vol_step_size       = MathRange(MathRound((data::position_list[pos_id].initial_volume / zones_total), inline_MathDigits(symbol_vol_min)), symbol_vol_min, symbol_vol_max);
        const int       vol_steps_total     = (int)(data::position_list[pos_id].initial_volume / vol_step_size);
        const int       vol_reductions      = init_tp_ticks - current_tp_ticks;
        const int       vol_steps_remaining = (int)(zones_total - vol_reductions);


    // Calculate current state

        const int       current_zone        = data::position_list[pos_id].profit_ticks / zone_size;
        const double    current_zone_low    = data::position_list[pos_id].open_price + (((zone_size * zone_set) * short_f) / digit_f);
        const double    current_zone_high   = data::position_list[pos_id].open_price + (((zone_size * (zone_set + 1)) * short_f) / digit_f);
        const double    zone_switch_low     = current_zone_low - (((zone_offset / digit_f) * ((zone_set > NULL) & 0x01) * short_f));
        const double    zone_switch_high    = current_zone_high + (((zone_offset / digit_f)* ((zone_set > NULL) & 0x01) * short_f));
        const double    price_low           = iLow(data::position_list[pos_id].symbol, PERIOD_M1, iLowest(data::position_list[pos_id].symbol, PERIOD_M1, MODE_LOW, pos_periods_age, NULL)); 
        const double    price_high          = iHigh(data::position_list[pos_id].symbol, PERIOD_M1, iHighest(data::position_list[pos_id].symbol, PERIOD_M1, MODE_HIGH, pos_periods_age, NULL));

        const bool      zone_switched_low   = (zone_set > current_zone) && ((vol_reductions < current_zone) || (zone_set == NULL));
        const bool      zone_switched_high  = (zone_set < current_zone) && ((vol_reductions < current_zone) || (zone_set == NULL));



    // Skip if zone has not changed
        //DBG_SOFT_BREAKPOINT_CONDITION(TimeCurrent() >= D'2021.01.11 19:17');

        if(!zone_switched_low && !zone_switched_high)
        { 
/*
            // Close position if crossing bb_h buffer with close
            if( (data::position_list[pos_id].type_sell)
             && (bb_h[data::wh_index] < data::close[data::wh_index]) )
            { data::position_list[pos_id].volume = NULL; }
    
            if( (!data::position_list[pos_id].type_sell)
             && (bb_h[data::wh_index] > data::close[data::wh_index]) )
            { data::position_list[pos_id].volume = NULL; }
*/        
            return(true); 
        }
        
        /*
    // DEBUG
        DBG_MSG_VAR(data::position_list[pos_id].ticket);
        DBG_MSG_VAR(volume_zones_max);
        DBG_MSG_VAR(tp_zones_total);
        DBG_MSG_VAR(zones_total);
        DBG_MSG_VAR(zone_size);
        DBG_MSG_VAR(zone_set);
        DBG_MSG_VAR(current_zone);
        DBG_MSG_VAR(vol_reductions);
//        DBG_MSG_VAR(volume_zone_size);
        DBG_MSG_VAR(vol_step_size);
        DBG_MSG_VAR(data::position_list[pos_id].initial_volume);
        DBG_MSG_VAR(data::position_list[pos_id].volume);
        DBG_MSG_VAR(data::position_list[pos_id].age_secs);
        DBG_MSG_VAR(data::position_list[pos_id].open_price);
        DBG_MSG_VAR(data::position_list[pos_id].stop_loss);
        DBG_MSG_VAR(data::position_list[pos_id].initial_stop_loss);
        DBG_MSG_VAR(data::position_list[pos_id].take_profit);
        DBG_MSG_VAR(data::position_list[pos_id].initial_take_profit);
        DBG_MSG_VAR(data::position_list[pos_id].profit_ticks);
        DBG_MSG_VAR(data::position_list[pos_id].type_sell);
    // DEBUG
*/
    // Local init
    
        bool vol_update = false;
    
    
    // Update zone config
        
        // Initial zone entry
        if(zone_set == NULL)
        { 
            // Update zone info
            data::position_list[pos_id].stop_loss = data::position_list[pos_id].open_price + (price_granularity * short_f); 
            data::position_list[pos_id].volume -= MathRange(MathRound(data::position_list[pos_id].volume / 2.0, inline_MathDigits(symbol_vol_min)), vol_step_size, data::position_list[pos_id].volume);
            vol_update = true;
        }

        // Switching zone up
        else 
        {
            // Switching zone up
            if(zone_switched_high)
            { 
                // Update zone info
                data::position_list[pos_id].stop_loss += (((price_granularity * (current_zone - zone_set)) * short_f)); 
                data::position_list[pos_id].volume -=vol_step_size;
                vol_update = true;
            }
            
            // Switching zone down
            else if(zone_switched_low)
            { 
                // Update zone info
                data::position_list[pos_id].stop_loss -= (((price_granularity * (zone_set - current_zone)) * short_f));
            }
        }        

        // Reduce volume when switching up
        if(vol_update)
        {
            data::position_list[pos_id].take_profit -= (price_granularity * short_f);
        }


        

/*
        DBG_MSG_VAR(data::position_list[pos_id].volume);
        DBG_MSG_VAR(vol_step_size);
        DBG_MSG_VAR(inline_MathDigits(symbol_vol_min));
        DBG_MSG_VAR(MathRound(data::position_list[pos_id].volume, inline_MathDigits(symbol_vol_min)));
        DBG_MSG_VAR(data::position_list[pos_id].stop_loss);
        DBG_MSG_VAR(data::position_list[pos_id].take_profit);
*/
    // Return false will register execution error
    err_SetUserError(EX_SUCCESS);
    return(true);
}



//+------------------------------------------------------------------+
//| OnStopLoss()                                                     |
//+------------------------------------------------------------------+
const bool OnStopLoss(const ulong deal_ticket)
{
    /*********************************************************************************************************************************************************/
    /*                                                                                                                                                       */
    /* OnStopLoss handler                                                                                                                                    */
    /*                                                                                                                                                       */
    /*    This function will be called whenever a stoploss has been triggered.                                                                               */
    /*                                                                                                                                                       */


    // Access data on positon
    //data::position_list[order_id];
//    DebugBreak();

    // Return
    err_SetUserError(EX_SUCCESS);
    return(true);
}



//+------------------------------------------------------------------+
//| OnTakeProfit()                                                   |
//+------------------------------------------------------------------+
const bool OnTakeProfit(const ulong deal_ticket)
{
    /*********************************************************************************************************************************************************/
    /*                                                                                                                                                       */
    /* OnTakeProfit handler                                                                                                                                  */
    /*                                                                                                                                                       */
    /*    This function will be called whenever a take profit has been triggered.                                                                            */
    /*                                                                                                                                                       */


    // Access data on positon
    //data::position_list[order_id];
//    DebugBreak();

    // Return
    err_SetUserError(EX_SUCCESS);
    return(true);
}



//
// END Custom Profit and StopLoss handlers 
//*********************************************************************************************************************************************************/









//
// END internal helpers
//*********************************************************************************************************************************************************/
#endif // EA_ONTICK_MQH_INCLUDED






